@@ -46,6 +46,10 @@
 #  error "include vdef.h before vrt.h"
 #endif
 
+#define VRT_MAJOR_VERSION	17U
+
+#define VRT_MINOR_VERSION	0U
+
 /***********************************************************************
  * Major and minor VRT API versions.
  *
@@ -53,6 +57,39 @@
  * Whenever something is deleted or changed in a way which is not
  * binary/load-time compatible, increment MAJOR version
  *
+ * 17.0 (2023-03-15)
+ *	VXID is 64 bit
+ *	[cache.h] http_GetRange() changed
+ *	exp_close added to struct vrt_backend_probe
+ *	VRT_new_backend() signature changed
+ *	VRT_new_backend_clustered() signature changed
+ *	authority field added to struct vrt_backend
+ *	release field added to struct vdi_methods
+ * 16.0 (2022-09-15)
+ *	VMOD C-prototypes moved into JSON
+ *	VRT_AddVDP() deprecated
+ *	VRT_AddVFP() deprecated
+ *	VRT_RemoveVDP() deprecated
+ *	VRT_RemoveVFP() deprecated
+ * 15.0 (2022-03-15)
+ *	VRT_r_req_transport() added
+ *	VRT_Assign_Backend() added
+ *	VRT_StaticDirector() added
+ *	enum lbody_e changed
+ *	- previous enum lbody_e values are defined as macros
+ *	The following functions changed to take `const char *, BODY`:
+ *	- VRT_l_beresp_body()
+ *	- VRT_l_resp_body()
+ *	BODY can either be a BLOB or a STRANDS, but only a STRANDS
+ *	can take a non-NULL const char * prefix. The changes to BODY
+ *	assignments doesn't break the ABI or the API.
+ *	TOSTRAND() and TOSTRANDS() macros added
+ *	[cache.h] enum sess_close replaced by struct stream_close
+ *	[cache.h] http_IsHdr() added
+ *	[cache_filter.h] vfp_init_f() changed to take a VRT_CTX
+ *	[cache_filter.h] vdp_init_f() changed to take a VRT_CTX
+ *	[cache_filter.h] VRT_AddFilter() added
+ *	[cache_filter.h] VRT_RemoveFilter() added
  * 14.0 (2021-09-15)
  *	VIN_n_Arg() no directly returns the directory name.
  *	VSB_new() and VSB_delete() removed
@@ -225,10 +262,6 @@
  *	vrt_acl type added
  */
 
-#define VRT_MAJOR_VERSION	14U
-
-#define VRT_MINOR_VERSION	0U
-
 /***********************************************************************/
 
 #include <stddef.h>		// NULL, size_t
@@ -240,6 +273,7 @@ struct http;
 struct lock;
 struct req;
 struct stevedore;
+struct stream_close;
 struct suckaddr;
 struct vcl;
 struct vcldir;
@@ -253,8 +287,11 @@ struct VSC_main;
 struct vsc_seg;
 struct vsl_log;
 struct vsmw_cluster;
+struct wrk_vpi;
 struct ws;
 
+typedef const struct stream_close *stream_close_t;
+
 /***********************************************************************
  * VCL_STRANDS:
  *
@@ -281,6 +318,12 @@ struct strands {
  */
 extern const struct strands *vrt_null_strands;
 
+/*
+ * Macros for VCL_STRANDS creation
+ */
+#define TOSTRAND(s)(&(struct strands){.n=1,.p=(const char *[1]){(s)}})
+#define TOSTRANDS(x, ...)(&(struct strands){.n=x,.p=(const char *[x]){__VA_ARGS__}})
+
 /***********************************************************************
  * VCL_BLOB:
  *
@@ -318,7 +361,7 @@ extern const struct vrt_blob *vrt_null_blob;
 typedef const struct vrt_acl *			VCL_ACL;
 typedef const struct director *			VCL_BACKEND;
 typedef const struct vrt_blob *			VCL_BLOB;
-typedef const char *				VCL_BODY;
+typedef const void *				VCL_BODY;
 typedef unsigned				VCL_BOOL;
 typedef int64_t					VCL_BYTES;
 typedef vtim_dur				VCL_DURATION;
@@ -351,7 +394,8 @@ struct vrt_ctx {
 	unsigned			syntax;
 	unsigned			vclver;
 	unsigned			method;
-	unsigned			*handling;
+
+	struct wrk_vpi			*vpi;
 
 	/*
 	 * msg is for error messages and exists only for
@@ -486,7 +530,6 @@ VCL_VOID VRT_call(VRT_CTX, VCL_SUB);
 VCL_STRING VRT_BACKEND_string(VCL_BACKEND);
 VCL_STRING VRT_BOOL_string(VCL_BOOL);
 VCL_STRING VRT_BLOB_string(VRT_CTX, VCL_BLOB);
-VCL_STRING VRT_CollectString(VRT_CTX, const char *p, ...);
 VCL_STRING VRT_INT_string(VRT_CTX, VCL_INT);
 VCL_STRING VRT_IP_string(VRT_CTX, VCL_IP);
 VCL_STRING VRT_REAL_string(VRT_CTX, VCL_REAL);
@@ -520,6 +563,7 @@ struct vrt_endpoint {
 #define VRT_BACKEND_FIELDS(rigid)				\
 	rigid char			*vcl_name;		\
 	rigid char			*hosthdr;		\
+	rigid char			*authority;		\
 	vtim_dur			connect_timeout;	\
 	vtim_dur			first_byte_timeout;	\
 	vtim_dur			between_bytes_timeout;	\
@@ -530,6 +574,7 @@ struct vrt_endpoint {
 	do {					\
 		DA(vcl_name);			\
 		DA(hosthdr);			\
+		DA(authority);			\
 		DN(connect_timeout);		\
 		DN(first_byte_timeout);		\
 		DN(between_bytes_timeout);	\
@@ -551,7 +596,8 @@ struct vrt_backend {
 	unsigned			exp_status;		\
 	unsigned			window;			\
 	unsigned			threshold;		\
-	unsigned			initial;
+	unsigned			initial;		\
+	unsigned			exp_close;
 
 #define VRT_BACKEND_PROBE_HANDLE()		\
 	do {					\
@@ -561,6 +607,7 @@ struct vrt_backend {
 		DN(window);			\
 		DN(threshold);			\
 		DN(initial);			\
+		DN(exp_close);			\
 	} while (0)
 
 struct vrt_backend_probe {
@@ -572,9 +619,9 @@ struct vrt_backend_probe {
 };
 
 /* Backend related */
-VCL_BACKEND VRT_new_backend(VRT_CTX, const struct vrt_backend *);
+VCL_BACKEND VRT_new_backend(VRT_CTX, const struct vrt_backend *, VCL_BACKEND);
 VCL_BACKEND VRT_new_backend_clustered(VRT_CTX,
-    struct vsmw_cluster *, const struct vrt_backend *);
+    struct vsmw_cluster *, const struct vrt_backend *, VCL_BACKEND);
 size_t VRT_backend_vsm_need(VRT_CTX);
 void VRT_delete_backend(VRT_CTX, VCL_BACKEND *);
 struct vrt_endpoint *VRT_Endpoint_Clone(const struct vrt_endpoint *vep);
@@ -606,10 +653,15 @@ VCL_STRING VRT_GetHdr(VRT_CTX, VCL_HEADER);
  */
 
 enum lbody_e {
-	LBODY_SET,
-	LBODY_ADD,
+	LBODY_SET_STRING,
+	LBODY_ADD_STRING,
+	LBODY_SET_BLOB,
+	LBODY_ADD_BLOB,
 };
 
+#define LBODY_SET LBODY_SET_STRING
+#define LBODY_ADD LBODY_ADD_STRING
+
 VCL_BYTES VRT_CacheReqBody(VRT_CTX, VCL_BYTES maxsize);
 
 VCL_STRING VRT_ban_string(VRT_CTX, VCL_STRING);
@@ -639,8 +691,9 @@ typedef VCL_BACKEND vdi_resolve_f(VRT_CTX, VCL_BACKEND);
 typedef int vdi_gethdrs_f(VRT_CTX, VCL_BACKEND);
 typedef VCL_IP vdi_getip_f(VRT_CTX, VCL_BACKEND);
 typedef void vdi_finish_f(VRT_CTX, VCL_BACKEND);
-typedef enum sess_close vdi_http1pipe_f(VRT_CTX, VCL_BACKEND);
+typedef stream_close_t vdi_http1pipe_f(VRT_CTX, VCL_BACKEND);
 typedef void vdi_event_f(VCL_BACKEND, enum vcl_event_e);
+typedef void vdi_release_f(VCL_BACKEND);
 typedef void vdi_destroy_f(VCL_BACKEND);
 typedef void vdi_panic_f(VCL_BACKEND, struct vsb *);
 typedef void vdi_list_f(VRT_CTX, VCL_BACKEND, struct vsb *, int, int);
@@ -656,6 +709,9 @@ struct vdi_methods {
 	vdi_getip_f			*getip;
 	vdi_finish_f			*finish;
 	vdi_event_f			*event;
+	// called by VRT_DelDirector: deref all backends
+	vdi_release_f			*release;
+	// when refcount goes 0
 	vdi_destroy_f			*destroy;
 	vdi_panic_f			*panic;
 	vdi_list_f			*list;
@@ -675,8 +731,10 @@ VCL_VOID VRT_SetChanged(VCL_BACKEND, VCL_TIME);
 VCL_BACKEND VRT_AddDirector(VRT_CTX, const struct vdi_methods *,
     void *, const char *, ...) v_printflike_(4, 5);
 void VRT_DisableDirector(VCL_BACKEND);
+void VRT_StaticDirector(VCL_BACKEND);
 VCL_BACKEND VRT_LookupDirector(VRT_CTX, VCL_STRING);
 void VRT_DelDirector(VCL_BACKEND *);
+void VRT_Assign_Backend(VCL_BACKEND *dst, VCL_BACKEND src);
 
 /***********************************************************************
  * vmod_priv related
